<?php

// Exit if accessed directly.
if ( !defined('ABSPATH') ) {
	exit;
}

if (!class_exists('SO_Admin_Notifications')) {

	Class SO_Admin_Notifications {

		function __construct() {

			add_filter( 'plugin_action_links_' . plugin_basename( SO_PLUGIN_FILE ), array( $this, 'plugin_action_links' ) );

			// Update footer text
			add_filter( 'admin_footer_text', array( $this , 'so_footer_text' ) );
			add_filter( 'update_footer', array( $this , 'so_update_footer_text' ), 99 );

			// Filter to add Quick Help Widget
			add_filter( 'sa_active_plugins_for_quick_help', array( $this, 'so_active_plugins_for_quick_help' ), 10, 2 );

			add_filter( 'sa_is_page_for_notifications', array( $this, 'is_page_for_notifications' ), 10, 2 );

			add_action( 'admin_init', array( $this, 'so_dismiss_admin_notice' ) );

			add_action( 'admin_notices', array( $this, 'so_show_ready_designs_notice' ) );

			$this->may_be_show_sa_in_app_offer();

		}

		public function plugin_action_links( $links ) {

			$action_links = array(
				'settings' => '<a href="' . admin_url( 'edit.php?post_type=smart_offers&page=so-settings' ) . '" title="' . esc_attr( __( 'View Smart Offers Settings', 'smart-offers' ) ) . '">' . __( 'Settings', 'smart-offers' ) . '</a>',
				'need_help' => '<a href="'. esc_url( add_query_arg( array( 'page' => 'so-faqs' ), 'edit.php?post_type=smart_offers' ) ) .'" title="' . __( 'FAQ\'s', 'smart-offers' ) . '">' . __( 'FAQ\'s', 'smart-offers' ) . '</a>',
			);

			return array_merge( $action_links, $links );

		}

		public function so_footer_text( $so_footer_text ) {

			global $post, $pagenow;

			if( !empty( $pagenow ) ) {
				if (
					( is_object( $post ) && $post->post_type == 'smart_offers' )
					||
					( !empty( $_GET['page'] ) && ( $_GET['page'] == 'so-settings' || $_GET['page'] == 'so-about' || $_GET['page'] == 'so-shortcode' || $_GET['page'] == 'so-faqs' ) )
				) {
					$so_footer_text = __( '<strong>Thank you for using <span style="color: #753d81;">Smart Offers</span>. A huge thank you from <span style="color: #753d81;">StoreApps</span>!</strong>', 'smart-offers' );
				}
			}

			return $so_footer_text;

		}

		public function so_update_footer_text( $so_text ) {

			global $post, $pagenow;

			$so_plugin_data = get_plugin_data( WP_PLUGIN_DIR.'/smart-offers/smart-offers.php' );
			$so_current_version = $so_plugin_data['Version'];

			if( !empty( $pagenow ) ) {
				if (
					( is_object( $post ) && $post->post_type == 'smart_offers' )
					||
					( !empty( $_GET['page'] ) && ( $_GET['page'] == 'so-settings' || $_GET['page'] == 'so-about' || $_GET['page'] == 'so-shortcode' || $_GET['page'] == 'so-faqs' ) )
				) {
					$so_text = sprintf( __( 'Smart Offers version: <strong><span style="color: #753d81;">%s</span></strong>', 'smart-offers' ), $so_current_version );
				
				}
			}

			return $so_text;

		}

		/**
		 * Filter to add Quick Help Widget on SO post type page and Settings page
		 */
		function so_active_plugins_for_quick_help( $active_plugins = array(), $upgrader = null ) {

			global $post, $pagenow;

			if( !empty( $pagenow ) ) {
				if (
					( is_object( $post ) && $post->post_type == 'smart_offers' )
					||
					( !empty( $_GET['page'] ) && ( $_GET['page'] == 'so-settings' || $_GET['page'] == 'so-about' || $_GET['page'] == 'so-shortcode' || $_GET['page'] == 'so-faqs' ) )
				) {
					$active_plugins['so'] = 'smart-offers';
				}
			} elseif ( array_key_exists( 'so', $active_plugins ) ) {
				unset( $active_plugins['so'] );
			}

			return $active_plugins;
		}

		/**
		 * To determine whether to show notification on a page or not
		 * 
		 * @param bool $bool
		 * @param mixed $upgrader
		 * 
		 * @return bool $bool
		 */
		public function is_page_for_notifications( $bool = false, $upgrader = null ) {

			$active_plugins = apply_filters( 'sa_active_plugins_for_quick_help', array(), $upgrader );
			if ( array_key_exists( $upgrader->sku, $active_plugins ) ) {
				return true;
			}

			return $bool;

		}

		public function so_dismiss_admin_notice() {

			if( isset($_GET['so_dismiss_admin_notice']) && $_GET['so_dismiss_admin_notice'] == '1' && isset($_GET['option_name']) ) {
				$option_name = sanitize_text_field($_GET['option_name']);
				if ( strpos($option_name, 'smart_offers_ready_designs_imported') !== false ) {
					update_option( $option_name, 'no', 'no' );
				} else {
					update_option( $option_name.'_smart_offers', 'no', 'no' );
				}
				$referer = wp_get_referer();
				wp_safe_redirect( $referer );
				exit();
			}

		}

		/**
		 * To show ready offer design notice
		 * Added since: v3.4.1. Start Date: 08.10.2018
		 */
		public function so_show_ready_designs_notice() {

			$so_ready_designs_imported = get_option ( 'smart_offers_ready_designs_imported' );
			if ( 'yes' == $so_ready_designs_imported || 'no' == $so_ready_designs_imported ) {
				return;
			}

			global $post, $pagenow;

			if( !empty( $pagenow ) ) {
				if ( ( is_object( $post ) && $post->post_type == 'smart_offers' ) || ( !empty( $_GET['page'] ) && ( $_GET['page'] == 'so-settings' ) ) ) {
					$current_user = wp_get_current_user();
					if ( ! $current_user->exists() ) {
						return;
					}
					$current_user_display_name = $current_user->display_name;
					?>
					<style>
						.so_design_notice {
							width: 45%;
							background-color: #FFF !important;
							margin-top: 1%;
							padding: 1.5em;
							box-shadow: 0 0 7px 0 rgba(0, 0, 0, .2);
							font-size: 1.1em;
							border: 0.5em solid #fd714f;
							margin: 0 auto;
							text-align: center;
						}
						.so-main-headline {
							font-size: 2em;
							font-weight: bold;
							padding-bottom: 1em;
							color: #f2555b;
						}
						.so-sub-headline {
							padding-bottom: 1em;
							font-size: 1.2em;
							color: #2d3e50;
							line-height: 1.3em;
						}
						.so-cta {
							margin-top: 1em;
						}
						.button-hero {
							color: #FFF!important;
							border-color: #03a025!important;
							background: #03a025 !important;
							box-shadow: 0 1px 0 #03a025;
							font-size: 1em;
							font-weight: bold;
						}
						.button-hero:hover {
							color: #FFF!important;
							background: #0AAB2E!important;
							border-color: #0AAB2E!important;
						}
						.so-no-cta {
							margin-top: 1em;
						}
						.so-no-cta-a {
							text-decoration: none;
							color: lightgrey;
						}
					</style>
					<div class="so_design_notice">
						<div class="so-container-1">
								<div class="so-main-headline"><?php _e( 'New Offer Designs  😊', 'smart-offers' ); ?></div>
								<div class="so-sub-headline"><?php echo sprintf(__( 'Hey %s,', 'smart-offers' ), $current_user_display_name ); ?></div>
								<div class="so-sub-headline"><?php echo __( 'We have created some amazing offer designs for you.', 'smart-offers' ) ?></div>
								<div class="so-sub-headline"><?php echo __( 'They are ready to use. So all you need to do is fill in the product & rules and you are good to go.', 'smart-offers' ) ?></div>
								<div class="so-cta">
									<a class="button button-hero" id="so-ready-offer-designs" href="<?php echo admin_url('edit.php?post_type=smart_offers&page=so-about&action=so-import'); ?>">
										<?php echo __( 'Click to view these offers', 'smart-offers' ); ?>
									</a>
								</div>
								<div class="so-no-cta">
									<a class="so-no-cta-a" href="?so_dismiss_admin_notice=1&option_name=smart_offers_ready_designs_imported"><?php echo __( 'Dismiss', 'smart-offers' ) ?></a>
								</div>
							</div>
						</div>
					</div>
					<?php
				}
			}
		}

		/**
		 * Function to show SA in app offer if any.
		 * Added since: v3.4.2. Start Date: 26.10.2018
		 */
		public function may_be_show_sa_in_app_offer() {

			$get_post_type = isset( $_GET['post_type'] ) ? sanitize_text_field( wp_unslash( $_GET['post_type'] ) ) : '';

			if ( ! class_exists( 'SA_In_App_Offer' ) ) {
				include_once SA_SO_PLUGIN_DIRPATH . '/sa-includes/class-sa-in-app-offer.php';

				$args = array(
					'file'           => SO_PLUGIN_FILE,
					'prefix'         => 'so',
					'option_name'    => 'sa_offer_halloween_2018',
					'campaign'       => 'sa_halloween_2018',
					'start'          => '2018-10-30',
					'end'            => '2018-11-02',
					'is_plugin_page' => ( $get_post_type == 'smart_offers' ) ? true : false, // WPCS: CSRF ok.
				);

				SA_In_App_Offer::get_instance( $args );
			}
		}

	}

	return new SO_Admin_Notifications();

}
